/**
 * WL socket header file
 *
 * Copyright (C) 2001-2002 Jeffrey Fulmer <jdfulmer@armstrong.com>
 * This file is part of libwacky
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef SOCK_H
#define SOCK_H

#include <wacky.h>

#ifdef  HAVE_SSL
# include <openssl/ssl.h>
# include <openssl/err.h>
# include <openssl/rsa.h>
# include <openssl/crypto.h>
# include <openssl/x509.h>
# include <openssl/pem.h>
#endif/*HAVE_SSL*/


/**
 * enum PROTOCOL
 * tcp/ip protocol
 */
typedef enum
{
  WACKY       = 0,
  WACKYS      = 1,
  UNSUPPORTED = 2
} PROTOCOL;  

typedef enum
{
  READ  = 0,
  WRITE = 1,
  RDWR  = 2
} SDSET; 

typedef struct
{
  int      sock;
  int      status;
  PROTOCOL prot;
#ifdef  HAVE_SSL
  SSL      *ssl;
  SSL_CTX  *ctx;
#endif/*HAVE_SSL*/
  fd_set   rset;
  fd_set   wset;
} CONN; 

/**
 * create socket 
 * const char *hostname
 * const char *service
 */
int WLsocket( CONN *conn, const char *hostname, int port );

/** 
 * write int sock 
 * from char *buf, 
 * unsigned int length 
 */
void WLsocket_write( CONN *conn, const void *b, size_t n );

/** 
 * read int sock, 
 * into void *buf, 
 * size_t length 
 */
ssize_t WLread_byte( CONN *conn, void *buf, size_t len ); 
ssize_t WLsocket_read( CONN *conn, void *buf, size_t len ); 

/** 
 * close int socket 
 */
void WLclose( CONN *C );

#endif/*SOCK_H*/

