/**
 * wackyp 
 *
 * Copyright (C) 2000 Jeffrey Fulmer <jdfulmer@armstrong.com>
 * This file is part of libwacky
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */  
#define  INTERN  1 

#include <setup.h>
#include <getopt.h>

/**
 * long options, std options struct
 */
static struct option long_options[] =
{
  { "version",    no_argument,       NULL, 'v' },
  { "help",       no_argument,       NULL, 'h' },
  { "config",     no_argument,       NULL, 'C' },
  { "pattern",    required_argument, NULL, 'p' },
  { "message",    required_argument, NULL, 'm' },
  { "priority",   required_argument, NULL, 'P' },
};

/**
 * display_version   displays the version number and exits
 *                   on boolean false.
 */
void
display_version( int i )
{
  /**
   * version_string is defined in version.c
   * adding it to a separate file allows us
   * to parse it in configure.
   */
  #ifdef DEBUG
  printf( "%s %s: debugging enabled\n", program_string, version_string );
  #else
  printf( "%s %s\n", program_string, version_string );
  #endif
  /**
   * if TRUE, exit
   */
  if( i == 1 ){ exit( 0 ); }
}  /* end of display version */


/**
 * display_help      displays the help section to STDOUT and exits
 */
void
display_help()
{
  /**
   * call display_version, but do not exit
   */
  display_version( FALSE );
  printf("Usage: %s [options] \n", program_string );
  printf("       %s [options] hostname\n", program_string );
  puts("       note:  you must specify a hostname if you do not use");
  puts("              $HOME/.wackyrc or if you want to override the ");
  puts("              value in that file.");
  printf("Options:\n"                    );
  puts("  -v, --version           VERSION, prints version number to screen.");
  puts("  -h, --help              HELP, prints this section.");
  puts("  -C, --config            CONFIGURATION, show the current configuration."); 
  puts("  -p, --pattern=xxx,ooo   PATTERN, coded pattern for wackyd to run." ); 
  puts("  -m, --message=MESSAGE   MESSAGE, send a message to be logged by wackyd.");
  puts("  -P, --priority=NUM      PRIORITY, set the priority of the notification.");
  /**
   * our work is done, exit nicely
   */
  exit(0);
}

/**
 * parse_cmdline      parses command line arguments and assigns
 *                    values to run time variables.
 */
void
parse_cmdline( int argc, char *argv[] )
{
  int c = 0, nhosts;

  while ((c = getopt_long (argc, argv, "vhCp:m:P:", long_options, (int *)0)) != EOF){
  switch( c ){
      case 'v':
        display_version( TRUE );
        break;
      case 'h':
        display_help();
        exit( 0 );
      case 'C':
        my.config = TRUE;
        break;
      case 'p':
        my.pattern = optarg;
        break;
      case 'm':
        my.message = optarg;
        break;
      case 'P':
        my.priority = atoi( optarg );
        break;
    } /** end of switch( c )           **/
  }   /** end of while c = getopt_long **/
  nhosts      = argc - optind;
  if( nhosts==0 && my.hostname == NULL ){ my.hostname="localhost"; } 
  else if( nhosts>0 ){ my.hostname = argv[argc-1]; }
}     /** end of parse_cmdline         **/

int
main( int argc, char *argv[] )
{
  char *hostname, *pattern;

  init_config(); 
  parse_cmdline(argc, argv);  

  my.pattern  = (my.pattern==NULL)?"xoo,oxo,oox,oxo":my.pattern;
  my.message  = (my.message==NULL)?"":my.message;
  my.priority = ( ! my.priority  )?0:my.priority;

  if( signal_wackyd( my.hostname, my.pattern, my.message, my.priority ) < 0 )
    fprintf( stderr, "unable to connect to server: %s\n", my.hostname );
 
  exit( 0 );
} 
